package com.sun.java.swing.plaf.windows;

import javax.swing.border.CompoundBorder;
import java.awt.Component;
import java.awt.Insets;
import javax.swing.border.Border;
import javax.swing.ButtonModel;
import javax.swing.JButton;
import javax.swing.JToolBar;
import javax.swing.JRadioButton;
import javax.swing.JCheckBox;
import java.awt.Dimension;
import javax.swing.plaf.basic.BasicGraphicsUtils;
import java.awt.Graphics;
import javax.swing.LookAndFeel;
import javax.swing.UIManager;
import javax.swing.AbstractButton;
import sun.awt.AppContext;
import javax.swing.plaf.ComponentUI;
import javax.swing.JComponent;
import java.awt.Rectangle;
import java.awt.Color;
import javax.swing.plaf.basic.BasicButtonUI;

public class WindowsButtonUI extends BasicButtonUI
{
    protected int dashedRectGapX;
    protected int dashedRectGapY;
    protected int dashedRectGapWidth;
    protected int dashedRectGapHeight;
    protected Color focusColor;
    private boolean defaults_initialized;
    private static final Object WINDOWS_BUTTON_UI_KEY;
    private Rectangle viewRect;
    
    public WindowsButtonUI() {
        this.defaults_initialized = false;
        this.viewRect = new Rectangle();
    }
    
    public static ComponentUI createUI(JComponent component) {
        AppContext appContext = AppContext.getAppContext();
        WindowsButtonUI value = (WindowsButtonUI)appContext.get(WindowsButtonUI.WINDOWS_BUTTON_UI_KEY);
        if (value == null) {
            value = new WindowsButtonUI();
            appContext.put(WindowsButtonUI.WINDOWS_BUTTON_UI_KEY, value);
            return value;
        }
        return value;
    }
    
    @Override
    protected void installDefaults(AbstractButton abstractButton) {
        super.installDefaults(abstractButton);
        if (!this.defaults_initialized) {
            String propertyPrefix = this.getPropertyPrefix();
            this.dashedRectGapX = UIManager.getInt(propertyPrefix + "dashedRectGapX");
            this.dashedRectGapY = UIManager.getInt(propertyPrefix + "dashedRectGapY");
            this.dashedRectGapWidth = UIManager.getInt(propertyPrefix + "dashedRectGapWidth");
            this.dashedRectGapHeight = UIManager.getInt(propertyPrefix + "dashedRectGapHeight");
            this.focusColor = UIManager.getColor(propertyPrefix + "focus");
            this.defaults_initialized = true;
        }
        XPStyle xp = XPStyle.getXP();
        if (xp != null) {
            abstractButton.setBorder(xp.getBorder(abstractButton, getXPButtonType(abstractButton)));
            LookAndFeel.installProperty(abstractButton, "rolloverEnabled", Boolean.TRUE);
        }
    }
    
    @Override
    protected void uninstallDefaults(AbstractButton b) {
        super.uninstallDefaults(b);
        this.defaults_initialized = false;
    }
    
    protected Color getFocusColor() {
        return this.focusColor;
    }
    
    @Override
    protected void paintText(Graphics graphics, AbstractButton abstractButton, Rectangle rectangle, String s) {
        WindowsGraphicsUtils.paintText(graphics, abstractButton, rectangle, s, this.getTextShiftOffset());
    }
    
    @Override
    protected void paintFocus(Graphics g, AbstractButton abstractButton, Rectangle rectangle, Rectangle rectangle2, Rectangle rectangle3) {
        int width = abstractButton.getWidth();
        int height = abstractButton.getHeight();
        g.setColor(this.getFocusColor());
        BasicGraphicsUtils.drawDashedRect(g, this.dashedRectGapX, this.dashedRectGapY, width - this.dashedRectGapWidth, height - this.dashedRectGapHeight);
    }
    
    @Override
    protected void paintButtonPressed(Graphics graphics, AbstractButton abstractButton) {
        this.setTextShiftOffset();
    }
    
    @Override
    public Dimension getPreferredSize(JComponent c) {
        Dimension preferredSize = super.getPreferredSize(c);
        AbstractButton abstractButton = (AbstractButton)c;
        if (preferredSize != null && abstractButton.isFocusPainted()) {
            if (preferredSize.width % 2 == 0) {
                ++preferredSize.width;
            }
            if (preferredSize.height % 2 == 0) {
                ++preferredSize.height;
            }
        }
        return preferredSize;
    }
    
    @Override
    public void paint(Graphics g, JComponent c) {
        if (XPStyle.getXP() != null) {
            paintXPButtonBackground(g, c);
        }
        super.paint(g, c);
    }
    
    static TMSchema.Part getXPButtonType(AbstractButton abstractButton) {
        if ((abstractButton instanceof JCheckBox)) {
            return TMSchema.Part.BP_CHECKBOX;
        }
        if ((abstractButton instanceof JRadioButton)) {
            return TMSchema.Part.BP_RADIOBUTTON;
        }
        return (abstractButton.getParent() instanceof JToolBar) ? TMSchema.Part.TP_BUTTON : TMSchema.Part.BP_PUSHBUTTON;
    }
    
    static TMSchema.State getXPButtonState(AbstractButton abstractButton) {
        TMSchema.Part xpButtonType = getXPButtonType(abstractButton);
        ButtonModel model = abstractButton.getModel();
        TMSchema.State state = TMSchema.State.NORMAL;
        switch (xpButtonType) {
            case BP_RADIOBUTTON:
            case BP_CHECKBOX: {
                if (!model.isEnabled()) {
                    state = (model.isSelected() ? TMSchema.State.CHECKEDDISABLED : TMSchema.State.UNCHECKEDDISABLED);
                    break;
                }
                if (model.isPressed() && model.isArmed()) {
                    state = (model.isSelected() ? TMSchema.State.CHECKEDPRESSED : TMSchema.State.UNCHECKEDPRESSED);
                    break;
                }
                if (model.isRollover()) {
                    state = (model.isSelected() ? TMSchema.State.CHECKEDHOT : TMSchema.State.UNCHECKEDHOT);
                    break;
                }
                state = (model.isSelected() ? TMSchema.State.CHECKEDNORMAL : TMSchema.State.UNCHECKEDNORMAL);
                break;
            }
            case BP_PUSHBUTTON:
            case TP_BUTTON: {
                if ((abstractButton.getParent() instanceof JToolBar)) {
                    if (model.isArmed() && model.isPressed()) {
                        state = TMSchema.State.PRESSED;
                        break;
                    }
                    if (!model.isEnabled()) {
                        state = TMSchema.State.DISABLED;
                        break;
                    }
                    if (model.isSelected() && model.isRollover()) {
                        state = TMSchema.State.HOTCHECKED;
                        break;
                    }
                    if (model.isSelected()) {
                        state = TMSchema.State.CHECKED;
                        break;
                    }
                    if (model.isRollover()) {
                        state = TMSchema.State.HOT;
                        break;
                    }
                    if (abstractButton.hasFocus()) {
                        state = TMSchema.State.HOT;
                        break;
                    }
                    break;
                }
                else {
                    if ((model.isArmed() && model.isPressed()) || model.isSelected()) {
                        state = TMSchema.State.PRESSED;
                        break;
                    }
                    if (!model.isEnabled()) {
                        state = TMSchema.State.DISABLED;
                        break;
                    }
                    if (model.isRollover() || model.isPressed()) {
                        state = TMSchema.State.HOT;
                        break;
                    }
                    if (abstractButton instanceof JButton && ((JButton)abstractButton).isDefaultButton()) {
                        state = TMSchema.State.DEFAULTED;
                        break;
                    }
                    if (abstractButton.hasFocus()) {
                        state = TMSchema.State.HOT;
                        break;
                    }
                    break;
                }
            }
            default: {
                state = TMSchema.State.NORMAL;
                break;
            }
        }
        return state;
    }
    
    static void paintXPButtonBackground(Graphics graphics, JComponent component) {
        AbstractButton abstractButton = (AbstractButton)component;
        XPStyle xp = XPStyle.getXP();
        TMSchema.Part xpButtonType = getXPButtonType(abstractButton);
        if (abstractButton.isContentAreaFilled() && xp != null) {
            XPStyle.Skin skin = xp.getSkin(abstractButton, xpButtonType);
            TMSchema.State xpButtonState = getXPButtonState(abstractButton);
            Dimension size = component.getSize();
            int n = 0;
            int n2 = 0;
            int width = size.width;
            int height = size.height;
            Border border = component.getBorder();
            Insets insets;
            if (border != null) {
                insets = getOpaqueInsets(border, component);
            }
            else {
                insets = component.getInsets();
            }
            if (insets != null) {
                n += insets.left;
                n2 += insets.top;
                width -= insets.left + insets.right;
                height -= insets.top + insets.bottom;
            }
            skin.paintSkin(graphics, n, n2, width, height, xpButtonState);
        }
    }
    
    private static Insets getOpaqueInsets(Border border, Component component) {
        if (border == null) {
            return null;
        }
        if (border.isBorderOpaque()) {
            return border.getBorderInsets(component);
        }
        if ((border instanceof CompoundBorder)) {
            CompoundBorder compoundBorder = (CompoundBorder)border;
            Insets opaqueInsets = getOpaqueInsets(compoundBorder.getOutsideBorder(), component);
            if (opaqueInsets != null && opaqueInsets.equals(compoundBorder.getOutsideBorder().getBorderInsets(component))) {
                Insets opaqueInsets2 = getOpaqueInsets(compoundBorder.getInsideBorder(), component);
                if (opaqueInsets2 == null) {
                    return opaqueInsets;
                }
                return new Insets(opaqueInsets.top + opaqueInsets2.top, opaqueInsets.left + opaqueInsets2.left, opaqueInsets.bottom + opaqueInsets2.bottom, opaqueInsets.right + opaqueInsets2.right);
            }
            else {
                return opaqueInsets;
            }
        }
        else {
            return null;
        }
    }
    
    static {
        WINDOWS_BUTTON_UI_KEY = new Object();
    }
}
